@@ -1,6 +1,6 @@
 /*****************************************************************************
    levelmeters.c - Stereo level meters
-   Copyright (C) 2000 by Jaroslav Kysela <perex@suse.cz>
+   Copyright (C) 2000 by Jaroslav Kysela <perex@perex.cz>
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -14,7 +14,7 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 ******************************************************************************/
 
 #include "envy24control.h"
@@ -200,57 +200,57 @@ gint level_meters_timeout_callback(gpointer data)
 	for (idx = 0; idx <= pcm_output_channels; idx++) {
 		get_levels(idx, &l1, &l2);
 		widget = idx == 0 ? mixer_mix_drawing : mixer_drawing[idx-1];
-		if (!GTK_WIDGET_VISIBLE(widget))
-			continue;
-		redraw_meters(idx, widget->allocation.width, widget->allocation.height, l1, l2);
-		gdk_draw_pixmap(widget->window,
-				widget->style->black_gc,
-				pixmap[idx],
-				0, 0,
-				0, 0,
-				widget->allocation.width, widget->allocation.height);	
-	}
-	if (view_spdif_playback) {
-		for (idx = MAX_PCM_OUTPUT_CHANNELS + 1; idx <= MAX_OUTPUT_CHANNELS + spdif_channels; idx++) {
-			get_levels(idx, &l1, &l2);
-			widget = idx == 0 ? mixer_mix_drawing : mixer_drawing[idx-1];
-			if (!GTK_WIDGET_VISIBLE(widget))
-				continue;
+		if (GTK_WIDGET_VISIBLE(widget) && (pixmap[idx] != NULL)) {
 			redraw_meters(idx, widget->allocation.width, widget->allocation.height, l1, l2);
 			gdk_draw_pixmap(widget->window,
 					widget->style->black_gc,
 					pixmap[idx],
 					0, 0,
 					0, 0,
-					widget->allocation.width, widget->allocation.height);	
+					widget->allocation.width, widget->allocation.height);
+		}
+	}
+	if (view_spdif_playback) {
+		for (idx = MAX_PCM_OUTPUT_CHANNELS + 1; idx <= MAX_OUTPUT_CHANNELS + spdif_channels; idx++) {
+			get_levels(idx, &l1, &l2);
+			widget = idx == 0 ? mixer_mix_drawing : mixer_drawing[idx-1];
+			if (GTK_WIDGET_VISIBLE(widget) && (pixmap[idx] != NULL)) {
+				redraw_meters(idx, widget->allocation.width, widget->allocation.height, l1, l2);
+				gdk_draw_pixmap(widget->window,
+						widget->style->black_gc,
+						pixmap[idx],
+						0, 0,
+						0, 0,
+						widget->allocation.width, widget->allocation.height);
+			}
 		}
 	}
 	for (idx = MAX_PCM_OUTPUT_CHANNELS + MAX_SPDIF_CHANNELS + 1; idx <= input_channels + MAX_PCM_OUTPUT_CHANNELS + MAX_SPDIF_CHANNELS; idx++) {
 		get_levels(idx, &l1, &l2);
 		widget = idx == 0 ? mixer_mix_drawing : mixer_drawing[idx-1];
-		if (!GTK_WIDGET_VISIBLE(widget))
-			continue;
-		redraw_meters(idx, widget->allocation.width, widget->allocation.height, l1, l2);
-		gdk_draw_pixmap(widget->window,
-				widget->style->black_gc,
-				pixmap[idx],
-				0, 0,
-				0, 0,
-				widget->allocation.width, widget->allocation.height);	
+		if (GTK_WIDGET_VISIBLE(widget) && (pixmap[idx] != NULL)) {
+			redraw_meters(idx, widget->allocation.width, widget->allocation.height, l1, l2);
+			gdk_draw_pixmap(widget->window,
+					widget->style->black_gc,
+					pixmap[idx],
+					0, 0,
+					0, 0,
+					widget->allocation.width, widget->allocation.height);
+		}
 	}
 	for (idx = MAX_PCM_OUTPUT_CHANNELS + MAX_SPDIF_CHANNELS + MAX_INPUT_CHANNELS + 1; \
 		    idx <= spdif_channels + MAX_PCM_OUTPUT_CHANNELS + MAX_SPDIF_CHANNELS + MAX_INPUT_CHANNELS; idx++) {
 		get_levels(idx, &l1, &l2);
 		widget = idx == 0 ? mixer_mix_drawing : mixer_drawing[idx-1];
-		if (!GTK_WIDGET_VISIBLE(widget))
-			continue;
-		redraw_meters(idx, widget->allocation.width, widget->allocation.height, l1, l2);
-		gdk_draw_pixmap(widget->window,
-				widget->style->black_gc,
-				pixmap[idx],
-				0, 0,
-				0, 0,
-				widget->allocation.width, widget->allocation.height);	
+		if (GTK_WIDGET_VISIBLE(widget) && (pixmap[idx] != NULL)) {
+			redraw_meters(idx, widget->allocation.width, widget->allocation.height, l1, l2);
+			gdk_draw_pixmap(widget->window,
+					widget->style->black_gc,
+					pixmap[idx],
+					0, 0,
+					0, 0,
+					widget->allocation.width, widget->allocation.height);
+		}
 	}
 	return TRUE;
 }
@@ -261,9 +261,15 @@ void level_meters_reset_peaks(GtkButton *button, gpointer data)
 
 void level_meters_init(void)
 {
+	int err;
+
 	snd_ctl_elem_value_malloc(&peaks);
-	snd_ctl_elem_value_set_interface(peaks, SND_CTL_ELEM_IFACE_MIXER);
+	snd_ctl_elem_value_set_interface(peaks, SND_CTL_ELEM_IFACE_PCM);
 	snd_ctl_elem_value_set_name(peaks, "Multi Track Peak");
+	if ((err = snd_ctl_elem_read(ctl, peaks)) < 0)
+		/* older ALSA driver, using MIXER type */
+		snd_ctl_elem_value_set_interface(peaks,
+			SND_CTL_ELEM_IFACE_MIXER);
 }
 
 void level_meters_postinit(void)
